#
# virtuozzo.py
#
# Copyright (c) 2015 Parallels IP Holdings GmbH
#
# This file is part of OpenVZ. OpenVZ is free software;
# you can redistribute it and/or modify it under the terms of the GNU
# Lesser General Public License as published by the Free Software Foundation;
# either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Our contact details: Parallels IP Holdings GmbH, Vordergasse 59, 8200
# Schaffhausen, Switzerland.

from pyanaconda.installclass import BaseInstallClass
from pyanaconda.constants import *
from pyanaconda.product import productName
from pyanaconda.flags import flags
from pyanaconda import network
from pyanaconda import nm
from pyanaconda.i18n import _
from pyanaconda.iutil import getSysroot
from pyanaconda.packaging.yumpayload import YumPayload
from pykickstart.constants import CLEARPART_TYPE_ALL
from blivet.partspec import PartSpec
from blivet.platform import platform
from blivet.devicelibs import swap
from blivet.size import Size

import os, types

class VirtuozzoBaseInstallClass(BaseInstallClass):
    # name has underscore used for mnemonics, strip if you dont need it
    id = "virtuozzo"
    name = N_("_Virtuozzo")
    sortPriority = 20000
    hidden = 0

    defaultFS = "ext4"

    bootloaderTimeoutDefault = 5

    installUpdates = True

    _l10n_domain = "comps"

    efi_dir = "virtuozzo"

    # up2date repos
    up2date_repos = []

    # Dispatcher first start scripts
    prl_disp_first_start_script_non_chroot = "/usr/libexec/prl_disp_first_start"
    prl_disp_first_start_script = getSysroot() + prl_disp_first_start_script_non_chroot
    prl_disp_first_start_header = "#!/bin/bash\n\n"
    prl_disp_first_start_footer = "# Cleanup\n" \
        "rm -f %s\n" \
        "exit 0\n" % prl_disp_first_start_script_non_chroot
    prl_disp_first_start_lines = []

    efiboot_size = Size("200MiB")
    biosboot_size = Size("1MiB")
    boot_size = Size("1GiB")
    slash_size = Size("12GiB")
    max_slash_size = Size("24GiB")
    min_ssd_size = Size("30GiB") # With /boot we require >= 32G SSDs
    min_vz_size = Size("30GiB")
    min_cs_size = Size("100GiB")
    swap_size = swap.swapSuggestion()
    bootreq_size = efiboot_size + biosboot_size + boot_size
    min_hdd_size = swap_size + min_vz_size + slash_size

    def prl_disp_first_start(self, lines):
        self.prl_disp_first_start_lines += lines
        with open(self.prl_disp_first_start_script, "w") as f:
            f.write(self.prl_disp_first_start_header)
            for line in self.prl_disp_first_start_lines:
                f.write(line + "\n")
            f.write(self.prl_disp_first_start_footer)
            f.close()
            os.chmod(self.prl_disp_first_start_script, 0755)

    def setDefaultPartitioning(self, storage):
        # Fill here only basic partitions for compatibility
        autorequests = [PartSpec(mountpoint="/", fstype="ext4",
                                 size=Size("20GiB"), maxSize=Size("50GiB"), grow=True,
                                 requiredSpace=Size("20GiB"), btr=False, lv=True, encrypted=False)]

        bootreqs = platform.setDefaultPartitioning()
        if bootreqs:
            autorequests.extend(bootreqs)

        swp = swap.swapSuggestion()
        autorequests.append(PartSpec(fstype="swap", size=swp, grow=False,
                                     lv=True, encrypted=True))

        for autoreq in autorequests:
            if autoreq.fstype is None:
                if autoreq.mountpoint == "/boot":
                    autoreq.fstype = storage.defaultBootFSType
                else:
                    autoreq.fstype = storage.defaultFSType

        storage.autoPartitionRequests = autorequests

    def configure(self, anaconda):
        BaseInstallClass.configure(self, anaconda)
        # Check for kickstart autopart - prohibit without clearpart --all
        if flags.automatedInstall and anaconda.ksdata.autopart.autopart and \
                anaconda.ksdata.clearpart.type != CLEARPART_TYPE_ALL or \
                len(anaconda.ksdata.clearpart.devices) > 0:
            raise RuntimeError("The autopart kickstart key is only used with " \
                "clearpart --all and without --list, --linux, or --none")
        anaconda.storage.vstorage_cs = anaconda.ksdata.vstorage.cs_join
        self.setDefaultPartitioning(anaconda.storage)
        # Enabled update repos for yum
        if anaconda.ksdata.up2date.enabled and isinstance(anaconda._payload, YumPayload):
            for repo in anaconda._payload._yum.repos.repos.values():
                if not repo.id.endswith("-updates"):
                    continue
                self.up2date_repos.append(repo.id)
                # Also, enable main repos
                mrepo = re.sub("-updates$", "", repo.id)
                for mainrepo in (mrepo, mrepo + "-base", mrepo + "-os"):
                    if mainrepo not in (r.id for r in anaconda._payload._yum.repos.repos.values()):
                        continue
                    self.up2date_repos.append(mainrepo)
                    break

    def setNetworkOnbootDefault(self, ksdata):
        if network.has_some_wired_autoconnect_device():
            return
        # choose the device used during installation
        # (ie for majority of cases the one having the default route)
        dev = network.default_route_device() \
              or network.default_route_device(family="inet6")
        if not dev:
            return
        # ignore wireless (its ifcfgs would need to be handled differently)
        if nm.nm_device_type_is_wifi(dev):
            return
        network.update_onboot_value(dev, "yes", ksdata)

    @property
    def is_free(self):
        return productName == "OpenVZ"

    @property
    def get_release_f(self):
        if self.is_free:
            return "/etc/openvz-release"
        return "/etc/virtuozzo-release"

    @property
    def get_eula(self):
        if self.is_free:
            return "/usr/share/licenses/openvz-release/NOTICE"
        return "/usr/share/licenses/virtuozzo-release/EULA"

    def __init__(self):
        BaseInstallClass.__init__(self)
